@@ -25,7 +25,6 @@
 #include <qdir.h>
 #include <qicon.h>
 #include <QList>
-#include <q3listview.h>
 #include <qmessagebox.h>
 #include <qpixmap.h>
 #include <qstatusbar.h>
@@ -34,10 +33,14 @@
 #include <QTime>
 #include <QToolBar>
 #include <QWhatsThis>
+#include <QtDebug>
 
-#include "aggregator.h"
 #include <wibble/operators.h>
 
+#include <ept/apt/apt.h>
+#include <ept/debtags/debtags.h>
+#include <ept/textsearch/textsearch.h>
+
 
 // NApplication
 #include "applicationfactory.h"
@@ -61,22 +64,19 @@
 // NXml
 #include "xmldata.h"
 
-// NWidget
-#include "customizablelistview.h"
-
 // NException
 #include "exception.h"
 
 #include "extalgorithm.h"
 
+
+// NPackageSearch
+#include "packagesearchaboutdlg.h"
+#include "packagedisplaywidget.h"
+
 // NUtil
 #include "helpers.h"
 
-#include "packagesearchaboutdlg.h"
-#include "packagelistviewitem.h"
-
-#include "eptinstance.h"
-
 
 using namespace std;
 
@@ -94,50 +94,57 @@ template class LesserPriority<NPlugin::ShortInformationPlugin>;
 PackageSearchImpl::PackageSearchImpl( QWidget* parent, const char* name, Qt::WFlags fl )
 : QMainWindow(parent, fl), _DETAILS_INFORMATION_PRIORITY(2)
 {
-	setName(name);
+	setObjectName(name);
 	setupUi(this);
+ 	delete _pPackageView;
+	_pPackageView = new NPackageSearch::PackageDisplayWidget(_pPackageViewFrame);
+ 	_pPackageViewFrame->layout()->addWidget(_pPackageView);
 	// former ui.h initialization
 	init();
 	_pPackageNamePlugin = 0;
 	_pScoreDisplayPlugin = 0;
-	_settingsFilename = QDir::homeDirPath()+"/.packagesearch";
-	_pInformationDetailsPage = _pInformationContainer->page(0);
+	_settingsFilename = QDir::homePath()+"/.packagesearch";
+	_pInformationDetailsPage = _pInformationContainer->widget(0);
 
-	_pPackageListViewControl = new NWidget::CustomizableListView(_pPackageView);
-	// this might be overwritten when loading the settings, but this is wanted
-	_pPackageListViewControl->setSorting("Score", Qt::Descending);
-	connect(_pPackageListViewControl, SIGNAL(shownColumnsChanged()), this, SLOT(onShortColumnsChanged()));
-	
 	vector<string> pluginDirectories;
 	pluginDirectories.push_back("plugins/");
 	pluginDirectories.push_back("/usr/lib/packagesearch/");
 	_pPluginManager = new NPlugin::PluginManager(pluginDirectories, this);
 	_pPluginManager->addPluginUser(this);
+	_pPluginManager->addPluginUser(packageView());
 	loadSettings();	// must be called after setting _pPluginManager but before calling loadPlugins
+	
+	_pApt = new ept::apt::Apt;
+	_pDebtags = new ept::debtags::Debtags;
+	_pTextsearch = new ept::textsearch::TextSearch;
 
 
 	QIcon backIcon(_iconDir + "back.png");
 	QIcon forwardIcon(_iconDir + "forward.png");
 	_pBackButton->setIcon(backIcon);
 	_pForwardButton->setIcon(forwardIcon);
-    connect(_pPackageView, SIGNAL(selectionChanged(Q3ListViewItem*)), this, SLOT(onPackageSelectionChanged(Q3ListViewItem*)));
-    connect(_pClearSearchButton, SIGNAL(clicked()), this, SLOT(onClearSearch()));
-    connect(helpAboutAction, SIGNAL(activated()), this, SLOT(onHelpAboutAction()));
-    connect(helpContentsAction, SIGNAL(activated()), this, SLOT(onHelpContentsAction()));
-    connect(_pBackButton, SIGNAL(clicked()), this, SLOT(onBack()));
-    connect(_pForwardButton, SIGNAL(clicked()), this, SLOT(onForward()));
-    connect(_pFileExitAction, SIGNAL(activated()), this, SLOT(close()));
-    connect(_pDetailsView, SIGNAL(sourceChanged(const QUrl&)), this, SLOT(onLinkClicked(const QUrl&)));
-    connect(_pInformationContainer, SIGNAL(currentChanged(QWidget*)), this, SLOT(onInformationPageChanged(QWidget*)));
-    connect(_pControlPluginsAction, SIGNAL(activated()), this, SLOT(onControlPlugins()));
-    connect(_pPluginSettingsAction, SIGNAL(activated()), this, SLOT(onPluginSettings()));
-    connect(_pPackageSelection, SIGNAL(activated(const QString&)), this, SLOT(selectNewPackage(const QString&)));
+    connect(_pPackageView, SIGNAL(packageSelected(QString)), SLOT(onPackageSelectionChanged(QString)));
+    connect(_pClearSearchButton, SIGNAL(clicked()), SLOT(onClearSearch()));
+    connect(helpAboutAction, SIGNAL(triggered()), SLOT(onHelpAboutAction()));
+    connect(helpContentsAction, SIGNAL(triggered()), SLOT(onHelpContentsAction()));
+    connect(_pBackButton, SIGNAL(clicked()), SLOT(onBack()));
+    connect(_pForwardButton, SIGNAL(clicked()), SLOT(onForward()));
+    connect(_pFileExitAction, SIGNAL(triggered()), SLOT(close()));
+    connect(_pDetailsView, SIGNAL(sourceChanged(const QUrl&)), SLOT(onLinkClicked(const QUrl&)));
+    connect(_pInformationContainer, SIGNAL(currentChanged(int)), SLOT(onInformationPageChanged()));
+    connect(_pControlPluginsAction, SIGNAL(triggered()), SLOT(onControlPlugins()));
+    connect(_pPluginSettingsAction, SIGNAL(triggered()), SLOT(onPluginSettings()));
+    connect(_pPackageSelection, SIGNAL(activated(const QString&)), SLOT(selectNewPackage(const QString&)));
 }
 
 PackageSearchImpl::~PackageSearchImpl()
 {
 	delete _pPackageNamePlugin;
 	delete _pScoreDisplayPlugin;
+	delete _pApt;
+	delete _pDebtags;
+	delete _pTextsearch;
+
 }
 
 
@@ -148,13 +155,13 @@ void PackageSearchImpl::init()
 		{
 			QDir docDir = QDir::current();
 			docDir.cd("../doc/");
-			_docDir = docDir.absPath()+"/";
+			_docDir = docDir.absolutePath()+"/";
 		}
 		else if ( QFile("doc/content.html").exists() )
 		{
 			QDir docDir = QDir::current();
 			docDir.cd("doc/");
-			_docDir = docDir.absPath()+"/";
+			_docDir = docDir.absolutePath()+"/";
 		}
 		else if ( QFile("/usr/share/doc/packagesearch/content.html").exists() )
 			_docDir = "/usr/share/doc/packagesearch/";
@@ -172,7 +179,7 @@ void PackageSearchImpl::init()
 				"/usr/share/pixmaps/packagesearch, ../icons/ or icons/."<<endl;
 	} // search the data paths
 	QPixmap appIcon(_iconDir+"packagesearch.png");
-	setIcon(appIcon);
+	setWindowIcon(appIcon);
 
 	_pWhatsThisAction = QWhatsThis::createAction(this);
 
@@ -202,7 +209,7 @@ void PackageSearchImpl::initToolbars()
 	_pWhatsThisAction->setShortcut(QKeySequence("Shift+F1"));
 	_pMainToolBar = addToolBar(tr("Main Toolbar"));
 	_pMainToolBar->setIconSize(QSize(16,16));
-	_pWhatsThisAction->addTo(_pMainToolBar);
+	_pMainToolBar->addAction(_pWhatsThisAction);
 	_nameToToolBar["Main"] = _pMainToolBar;
 }
 
@@ -230,13 +237,17 @@ void PackageSearchImpl::initialize()
 	// create the hard coded plugins
 	_pPackageNamePlugin = new NPlugin::PackageNamePlugin();
 	this->addPlugin(_pPackageNamePlugin);
+	packageView()->addPlugin(_pPackageNamePlugin);
 	
-	_pScoreDisplayPlugin = new NPlugin::ScoreDisplayPlugin(this);
+	_pScoreDisplayPlugin = new NPlugin::ScoreDisplayPlugin();
 	_pPluginManager->addPluginUser(_pScoreDisplayPlugin);
 	this->addPlugin(_pScoreDisplayPlugin);
+	packageView()->addPlugin(_pScoreDisplayPlugin);
 	
 	_pPluginManager->loadPlugins();
 	
+	packageView()->initialize();
+
 	// remove all columns for plugins which are not loaded
 	{
 		QStringList columns;
@@ -245,9 +256,9 @@ void PackageSearchImpl::initialize()
 		{
 			columns.push_back((*it)->shortInformationCaption());
 		}
-		_pPackageListViewControl->removeColumnsNotInList(columns);
+// 		_pPackageListViewControl->removeColumnsNotInList(columns);
 	}
-	_pInformationContainer->setCurrentPage(0);
+	_pInformationContainer->setCurrentIndex(0);
 	const set<string>& packages_ = packages();
 	// First copy the package names into a stringlist, and then add the whole list
 	// to the combobox, because since QT4 _pPackageSelection->insertItem() is totally inefficient
@@ -258,7 +269,6 @@ void PackageSearchImpl::initialize()
 	}
 	_pPackageSelection->insertItems(0, packageNames);
 	_pPackageSelection->setMinimumWidth(100);
-	_pPackageSelection->setCurrentText("");
 	setPackageActionsEnabled(false);
 }
 
@@ -289,7 +299,7 @@ void PackageSearchImpl::saveSettings()
 	xmlData.addAttribute(window, sizes[0], "lowerVSplitterSize1");
 	xmlData.addAttribute(window, sizes[1], "lowerVSplitterSize2");
 
-	_pPackageListViewControl->saveSettings(xmlData, root);
+ 	packageView()->saveSettings(xmlData, root);
 	_pPluginManager->saveSettings(xmlData, root);
 	if ( !xmlData.writeFile(_settingsFilename) )
 		reportError(tr("Unable to write settings"), tr("Unable to write settings to") +
@@ -302,7 +312,7 @@ void PackageSearchImpl::loadSettings()
 	if (!xmlData.loadFile(_settingsFilename))
 	{
 		// sort by the first column by default
-		_pPackageView->setSorting(1, false);
+// 		_pPackageView->setSorting(1, false);
 		return;
 	}
 	QDomElement root = xmlData.root();
@@ -352,7 +362,7 @@ void PackageSearchImpl::loadSettings()
 	// <customListView> node since version 0.3
 	if (settingsVersion >= QString("0.3"))
 	{
-		element = _pPackageListViewControl->loadSettings(element);
+		element = packageView()->loadSettings(element);
 	}
 	// dismiss old setting version
 	else
@@ -382,10 +392,11 @@ void PackageSearchImpl::setEnabled(bool enabled)
 
 QPushButton* PackageSearchImpl::createClearButton(QWidget* pParent, const char* name) const
 {
-	QPushButton* pButton = new QPushButton(pParent, name);
+	QPushButton* pButton = new QPushButton(pParent);
+	pButton->setObjectName(name);
 	pButton->setFixedSize(20, 20);
 	QPixmap clearIcon(_iconDir+"clear.png");
-	pButton->setPixmap(clearIcon);
+	pButton->setIcon(clearIcon);
 	return pButton;
 }
 
@@ -399,16 +410,16 @@ void PackageSearchImpl::reportWarning(const QString& title, const QString& messa
 	QMessageBox::warning(this, title, message, "OK");
 }
 
-void PackageSearchImpl::reportBusy(NPlugin::Plugin* pPlugin, const QString& message)
+void PackageSearchImpl::reportBusy(NPlugin::Plugin*, const QString& message)
 {
 	qApp->setOverrideCursor(QCursor(Qt::WaitCursor));
-	statusBar()->message(message);
+	statusBar()->showMessage(message);
 }
 
-void PackageSearchImpl::reportReady(NPlugin::Plugin* pPlugin)
+void PackageSearchImpl::reportReady(NPlugin::Plugin* )
 {
 	qApp->restoreOverrideCursor();
-	statusBar()->message(tr("Ready"));
+	statusBar()->showMessage(tr("Ready"));
 }
 
 
@@ -438,18 +449,16 @@ const set<string>& PackageSearchImpl::packages() const
 	return _packages;
 }
 
-
-NPlugin::IProvider::Aggregator& PackageSearchImpl::aptFrontCache() const
-{
-	return *EptInstance::aggregator();
-}
-
 void PackageSearchImpl::reloadAptFrontCache()
 {
-	aptFrontCache().invalidate();
+	delete _pDebtags;
+	delete _pApt;
+	delete _pTextsearch;
+	_pApt = new ept::apt::Apt;
+	_pDebtags = new ept::debtags::Debtags;
+	_pTextsearch = new ept::textsearch::TextSearch;
 }
 
-
 /////////////////////////////////////////////////////
 // IPluginUser Interface
 /////////////////////////////////////////////////////
@@ -496,7 +505,7 @@ void PackageSearchImpl::addPlugin(NPlugin::PluginContainer* pPlugin)
 
 void PackageSearchImpl::removePlugin(NPlugin::Plugin* pPlugin)
 {
-	qDebug("Removing plugin " + pPlugin->name());
+	qDebug() << "Removing plugin " << pPlugin->name();
 	using namespace NPlugin;
 	SearchPlugin* pSPlugin = dynamic_cast<SearchPlugin*>(pPlugin);
 	if (pSPlugin)
@@ -514,7 +523,7 @@ void PackageSearchImpl::removePlugin(NPlugin::Plugin* pPlugin)
 	if (pSIPlugin)
 	{
 		_shortInformationPlugins.erase(pSIPlugin);
-		_pPackageListViewControl->removeColumn(pSIPlugin->shortInformationCaption());
+// 		_pPackageListViewControl->removeColumn(pSIPlugin->shortInformationCaption());
 	}
 	ActionPlugin* pAPlugin = dynamic_cast<ActionPlugin*>(pPlugin);
 	if (pAPlugin)
@@ -534,12 +543,10 @@ void PackageSearchImpl::removePlugin(NPlugin::Plugin* pPlugin)
 
 void PackageSearchImpl::addShortInformationPlugin(NPlugin::ShortInformationPlugin* pPlugin)
 {
-	const int CHAR_WIDTH = 7;
-	const int MARGIN = 10;
 	_shortInformationPlugins.insert(pPlugin);
 	// if the column is not already in the list, add it before the first column
 	// with a higher priority value
-	if (!_pPackageListViewControl->isExisting(pPlugin->shortInformationCaption()))
+/*	if (!_pPackageListViewControl->isExisting(pPlugin->shortInformationCaption()))
 	{
 		QStringList columns = _pPackageListViewControl->shownColumns();
 		int position = 0;
@@ -560,7 +567,7 @@ void PackageSearchImpl::addShortInformationPlugin(NPlugin::ShortInformationPlugi
 			position,
 			(pPlugin->preferredColumnWidth()==-1) ? -1 : pPlugin->preferredColumnWidth() * CHAR_WIDTH + MARGIN
 		);
-	}
+	}*/
 }
 
 void PackageSearchImpl::addActionPlugin(NPlugin::ActionPlugin* pPlugin)
@@ -579,8 +586,10 @@ void PackageSearchImpl::addActionPlugin(NPlugin::ActionPlugin* pPlugin)
 			}
 			else
 			{
-				qDebug("Requested to add to an unknown menu \"%s\" in plugin %s.\n"
-					"Added to system menu instead.", pAction->menu().ascii(), pPlugin->name().ascii() );
+				QString debugMessage = "Requested to add to an unknown menu \"" 
+					+ pAction->menu() + "\" in plugin " + pPlugin->name() + ".\n"
+					+ "Added to system menu instead.";
+				qDebug(debugMessage.toAscii());
 				pMenu = _pSystemMenu;
 			}
 			pMenu->addAction(pAction->action());
@@ -595,8 +604,10 @@ void PackageSearchImpl::addActionPlugin(NPlugin::ActionPlugin* pPlugin)
 			}
 			else
 			{
-				qDebug("Requested to add to an unknown toolbar \"%s\" in plugin %s.\n"
-					"Added to main toolbar instead.", pAction->toolBar().ascii(), pPlugin->name().ascii() );
+				QString debugMessage = "Requested to add to an unknown toolbar \""
+					+ pAction->toolBar() + "\" in plugin " + pPlugin->name() + "\n"
+					+ "Added to main toolbar instead.";
+				qDebug(debugMessage.toAscii());
 				pToolBar = _pMainToolBar;
 			}
 			pToolBar->addAction(pAction->action());
@@ -613,30 +624,28 @@ void PackageSearchImpl::addActionPlugin(NPlugin::ActionPlugin* pPlugin)
 // Helper Methods
 /////////////////////////////////////////////////////
 
+NPackageSearch::PackageDisplayWidget* PackageSearchImpl::packageView()
+{
+	return static_cast<NPackageSearch::PackageDisplayWidget*>(_pPackageView);
+}
+
 void PackageSearchImpl::updateSearchPluginGui()
 {
 	// remove all pages
-	while ( _pInputWidgetsContainer->count() > 0)
-	{
-		_pInputWidgetsContainer->removePage(_pInputWidgetsContainer->page(0) );
-	}
+	_pInputWidgetsContainer->clear();
 	
-#warning QT4 Transition todo
-// TODO fix this!! -> I only comented out the next two lines.
-	// delete the old layout for the frame
-//	delete _pShortSearchFrameLayout;
-// 	// replace by a new one
-//	_pShortSearchFrameLayout = new QVBoxLayout( _pShortSearchFrame, 5, 10, "_pShortSearchFrameLayout"); 
-	_pShortSearchFrame->setLayout(new QVBoxLayout());
-
+	// the next lines create a new layout for the _pShortSearchFrame so
+	// to have an empty frame so adding the information widgets will result in the
+	// correct order
+	delete _pShortSearchFrame->layout();
+	QVBoxLayout* pLayout = new QVBoxLayout(_pShortSearchFrame);
+	pLayout->setMargin(5);
+	pLayout->setSpacing(10);
 	
-	//_pShortSearchFrame->layout()
 	NExtStd::for_each(_searchPlugins.begin(), _searchPlugins.end(), 
 		&PackageSearchImpl::addSearchPluginToGui, this);
-	/// @todo this seemed to be neccessary because the tab widget behaved really odd
-	_pInputWidgetsContainer->setCurrentPage(1);
 	if (_pInputWidgetsContainer->count()>0)
-		_pInputWidgetsContainer->setCurrentPage(0);
+		_pInputWidgetsContainer->setCurrentIndex(0);
 }
 
 void PackageSearchImpl::addSearchPluginToGui(NPlugin::SearchPlugin* pPlugin)
@@ -652,13 +661,11 @@ void PackageSearchImpl::addSearchPluginToGui(NPlugin::SearchPlugin* pPlugin)
 	{
 		QWidget* pWidget = pPlugin->shortInputAndFeedbackWidget();
 		// store if the widget was shown to as reparenting destroys this state
-		bool shown = pWidget->isShown();
+ 		bool hidden = pWidget->isHidden();
 		// reparenting was the only way which seemed to work to get the widgets where I wanted them :-(
-		pWidget->reparent(
-			_pShortSearchFrame, 0, QPoint()
-		);
-		_pShortSearchFrame->layout()->add(pWidget);
-		pWidget->setShown(shown);
+		pWidget->setParent(_pShortSearchFrame);
+		_pShortSearchFrame->layout()->addWidget(pWidget);
+		pWidget->setHidden(hidden);
 	}
 }
 
@@ -666,11 +673,8 @@ void PackageSearchImpl::updateInformationPluginGui()
 {
 	QString tmpCurrentPackage = _currentPackage;
 	_currentPackage =  "";
-	// clear the information plugin GUI except the first page
-	while ( _pInformationContainer->count() > 0)
-	{
-		_pInformationContainer->removePage(_pInformationContainer->page(0) );
-	}
+	// clear the information plugin GUI 
+	_pInformationContainer->clear();
 	// add a dummy tab, so that the widgets are not always repainted
 	bool detailPageAdded = false;
 	for (InformationPluginContainer::iterator it = _informationPlugins.begin();
@@ -679,14 +683,14 @@ void PackageSearchImpl::updateInformationPluginGui()
 		if ( !detailPageAdded && 
 			((*it)->informationPriority() >= _DETAILS_INFORMATION_PRIORITY) )
 		{
-			_pInformationContainer->insertTab(_pDetailsView, tr("Details"));
+			_pInformationContainer->addTab(_pDetailsView, tr("Details"));
 			detailPageAdded = true;
 		}
 		addInformationPluginToGui(*it);
 	}
 	if (!detailPageAdded)
-		_pInformationContainer->insertTab(_pDetailsView, tr("Details"));
-	_pInformationContainer->showPage(_pInformationContainer->page(0));
+		_pInformationContainer->addTab(_pDetailsView, tr("Details"));
+	_pInformationContainer->setCurrentIndex(0);
 	setSelectedPackage(tmpCurrentPackage);
 /*	{
 		QWidget* pParent = dynamic_cast<QWidget*>(_pInformationContainer->parent());
@@ -715,7 +719,7 @@ void PackageSearchImpl::addInformationPluginToGui(NPlugin::InformationPlugin* pP
 	if (pPlugin->informationWidget() != 0)	// if there is a special information widget
 	{
 		// insert the widget
-		_pInformationContainer->insertTab(
+		_pInformationContainer->addTab(
 			pPlugin->informationWidget(), 
 			pPlugin->informationWidgetTitle()
 		);
@@ -734,12 +738,6 @@ NPlugin::ShortInformationPlugin* PackageSearchImpl::getShortInformationPluginWit
 		return *it;
 }
 
-void PackageSearchImpl::onShortColumnsChanged()
-{
-	onSearchChanged(0);
-}
-
-
 void PackageSearchImpl::onClearSearch()
 {
 	for (
@@ -757,13 +755,14 @@ bool PackageSearchImpl::FilterPackages::operator()(const string& package)
 }
 
 
-void PackageSearchImpl::onSearchChanged(NPlugin::SearchPlugin* pPlugin)
+void PackageSearchImpl::onSearchChanged(NPlugin::SearchPlugin*)
 {
 	reportBusy(0, tr("Evaluating searches"));
 	using namespace NPlugin;
 	using namespace wibble::operators;
 	std::set<string> result;
 	bool first = true;	// keeps track if this is the first search which produces results
+	
 	// evaluate plugins which do not uses filter technique
 	for (SearchPluginContainer::iterator it = _searchPlugins.begin(); it != _searchPlugins.end(); ++it)
 	{
@@ -811,90 +810,26 @@ void PackageSearchImpl::onSearchChanged(NPlugin::SearchPlugin* pPlugin)
 	if (!first)
 	{
 		setPackagesFound(result.size());
+		if (packageView()->scoreColumnVisible())
+			_pScoreDisplayPlugin->updateScores(result);
 	}
 	else
 	{
 		setPackagesFound(-1);
 		result.clear();
 	}
-	updateShortInformation(result);
+	packageView()->setPackages(result);
 	reportReady(0);
 }
 
-
-void PackageSearchImpl::updateShortInformation(const set<string>& packages)
-{
-	string lastPackage;	// it seems like -1 is no valid package id (or seldom used..)
-	// if a package is selected
-	if ( !_currentPackage.isEmpty() )
-		lastPackage = toString(_currentPackage);
-	// this pointer holds the new item for the formerly selected package, 
-	// 0 if the package is nots in the new result
-	Q3ListViewItem* pSelectItem = 0;
-	_pPackageView->clear();
-	
-	
-	// collect all shown plugins in a list in order of appearance
-	// this holds all shown ShortInformationPlugins in order of appearance 
-	// and the section as second entry
-	vector< pair<NPlugin::ShortInformationPlugin*, int> > siPlugins;
-	const QStringList columns = _pPackageListViewControl->shownColumns();
-	int i = 0;
-	for (QStringList::const_iterator it = columns.begin(); it != columns.end(); ++it, ++i)
-	{
-		// if the current column is the score column calculate the scores, this way they are only
-		// calculated if the column is not hidden
-		if ( _pScoreDisplayPlugin != 0 && _pScoreDisplayPlugin->shortInformationCaption() == *it)
-			_pScoreDisplayPlugin->updateScores(packages);
-
-		
-		for (ShortInformationPluginContainer::iterator jt = _shortInformationPlugins.begin(); 
-			jt != _shortInformationPlugins.end(); ++jt)
-		{
-			if ((*jt)->shortInformationCaption() == *it)
-				siPlugins.push_back( make_pair(*jt, _pPackageListViewControl->mapToSection(i)) );
-		}
-	}
-	
-	for (set<string>::const_iterator it = packages.begin(); it != packages.end(); ++it)
-	{
-		const string& package = *it;
-		// add the item with its name
-		Q3ListViewItem* pItem = new PackageListViewItem(_pPackageView, toQString(package));
-		if (lastPackage == package)
-			pSelectItem = pItem;
-		// iterate all shown columns
-		for ( int i=0; i < siPlugins.size(); ++i)
-		{
-			try 
-			{
-				pItem->setText( siPlugins[i].second, siPlugins[i].first->shortInformationText(package) );
-			}
-			// simply ignore it if the package was not available for this plugin
-			catch (NPlugin::PackageNotFoundException& e) {}
-		}
-	}
-	if (pSelectItem)
-	{
-		_pPackageView->ensureItemVisible(pSelectItem);
-		_pPackageView->setSelected(pSelectItem, true);
-	}
-	// select the first item of the site to be shown
-	else if (_pPackageView->firstChild())
-	{
-		_pPackageView->setSelected(_pPackageView->firstChild(), true);
-	}
-
-}
-
 void PackageSearchImpl::showPackageInformation(const QString & package)
 {
-	// if the package to be shown is not selected, deselect it
-	PackageListViewItem* pSelected = dynamic_cast<PackageListViewItem*>(_pPackageView->selectedItem());
-	
-	if ( pSelected && package != pSelected->package())
-		_pPackageView->clearSelection();
+	// if the package to be shown is not the one selected in the package view, deselect it in the package view
+	if (packageView()->selectedPackage() != package)
+		packageView()->clearSelection();
 	_pDetailsView->clear();
+	
+	QString detailedText;	// detailed information text (in HTML)
 	for ( InformationPluginContainer::iterator it = _informationPlugins.begin();
 		it != _informationPlugins.end(); ++it )
 	{
@@ -909,31 +844,67 @@ void PackageSearchImpl::showPackageInformation(const QString & package)
 			{
 				try 
 				{
-					_pDetailsView->setText(_pDetailsView->text() + pPlugin->informationText(toString(package)));
+					detailedText += pPlugin->informationText(toString(package));
 				}
 				// simply ignore it if the package was not available for this plugin
 				catch (NPlugin::PackageNotFoundException& e) {}
+				catch (std::exception& e) {
+					qWarning() << "Plugin " << (*it)->name() << " failed to provide detailed information\n"
+							<< "e.what(): " << e.what();
+				}
+				catch (...) {
+					qWarning() << "Plugin " << (*it)->name() << " failed to provide detailed information (unknown exception thrown)";
+				}
+			}
+			if (_pInformationContainer->currentWidget() == pPlugin->informationWidget())
+			{
+			// update the tab only if it is currenly active
+				try  {
+					pPlugin->updateInformationWidget(toString(package));
+				}
+				catch (std::exception& e) {
+					qWarning() << "Plugin " << (*it)->name() << " failed to update information\n"
+							<< "e.what(): " << e.what();
+				}
+				catch (...) {
+					qWarning() << "Plugin " << (*it)->name() << " failed to update information (unknown exception thrown)";
+				}
 			}
-			if (_pInformationContainer->currentPage() == pPlugin->informationWidget())
-			// show the page only if it is currenly active
-				pPlugin->updateInformationWidget(toString(package));
 		}
 	}
+	_pDetailsView->setHtml("<p>" + detailedText + "<p>");
+// 	// Begin Debug
+// 	static QTextEdit* p = new QTextEdit();
+// 	p->show();
+// 	p->setPlainText(_pDetailsView->toHtml());
+// 	// End Debug
 }
 
-void PackageSearchImpl::onInformationPageChanged( QWidget * pPage )
+void PackageSearchImpl::onInformationPageChanged()
 {
 	if (_currentPackage.isEmpty())	// if no package was selected
 		return;
+	
 	for (
 		InformationPluginContainer::iterator it = _informationPlugins.begin();
 		it != _informationPlugins.end();
 		++it
 	)
 	{
-		if (_pInformationContainer->currentPage() == (*it)->informationWidget())
+		if (_pInformationContainer->currentWidget() == (*it)->informationWidget())
 		// show the page only if it is currenly active
-			(*it)->updateInformationWidget(toString(_currentPackage));
+		{
+			try  {
+				(*it)->updateInformationWidget(toString(_currentPackage));
+			}
+			catch (std::exception& e) {
+				qWarning() << "Plugin " << (*it)->name() << " failed to update information\n"
+						<< "e.what(): " << e.what();
+			}
+			catch (...) {
+				qWarning() << "Plugin " << (*it)->name() << " failed to update information (unknown exception thrown)";
+			}
+		}
 	}
 }
 
@@ -956,27 +927,11 @@ void PackageSearchImpl::closeEvent(QCloseEvent* pE)
 }
 
 
-void PackageSearchImpl::on__pPackageView_contextMenuRequested(Q3ListViewItem* pItem, const QPoint& pos)
+void PackageSearchImpl::onPackageSelectionChanged(QString package)
 {
-	QList<QAction*> actions;
-	
-	QAction* pCustomizeColumnsAction = new QAction(tr("Customize columns"), this);
-	actions.push_back(pCustomizeColumnsAction);
-	
-	if (pItem)
-		NExtStd::for_each(_packageActions.begin(), _packageActions.end(), &QList<QAction*>::push_back, &actions);
-
-	QAction* pSelectedAction = QMenu::exec(actions, pos);
-	if ( pSelectedAction == pCustomizeColumnsAction)
-		_pPackageListViewControl->showControlDialog();
-}
-
-void PackageSearchImpl::onPackageSelectionChanged(Q3ListViewItem* pItem)
-{
-	if ( pItem != 0 )
+	if ( !package.isNull() )
 	{
-		PackageListViewItem* pPackageItem = dynamic_cast<PackageListViewItem*>(pItem) ;
-		selectNewPackage(pPackageItem->package());
+		selectNewPackage(package);
 		setPackageActionsEnabled(true);
 	}
 	else
@@ -1025,9 +980,9 @@ void PackageSearchImpl::onHelpContentsAction()
 	QDialog* pDlg = new QDialog(this);
 	QVBoxLayout* pMainLayout = new QVBoxLayout(pDlg);
 	QTextBrowser *pHelpWindow = new QTextBrowser(pDlg); 
-	pHelpWindow->setCaption(tr("Package Search Help Page"));
+	pHelpWindow->setWindowTitle(tr("Package Search Help Page"));
 	pHelpWindow->setSource(_docDir+"content.html");
-	pMainLayout->add(pHelpWindow);
+	pMainLayout->addWidget(pHelpWindow);
 	pDlg->resize(520,560);
 	pDlg->show();
 }
@@ -1056,7 +1011,9 @@ void PackageSearchImpl::setSelectedPackage( const QString & package )
 		_currentPackage = package;
 		// this triggers a selectNewPackage() call  but it returns immidiately because 
 		// package == _currentPackage
-		_pPackageSelection->setCurrentText(package);	
+		int i = _pPackageSelection->findText(package);
+		if (i != -1)
+			_pPackageSelection->setCurrentIndex(i);
 	}
 	// reselect the package even if already selected to reflect changes due to a changed search
 	showPackageInformation(package);
@@ -1069,12 +1026,5 @@ void PackageSearchImpl::setPackageActionsEnabled(bool enabled)
 		(*it)->setEnabled(enabled);
 }
 
-// Include template instantiations as well
-// Need to undef emit because it's been defined by qt as something else
-#undef emit
-//#include <ept/cache/apt/packages.tcc>
-//#include <ept/cache/debtags/vocabulary.tcc>
-// Luckily, qt defines emit as an empty symbol, so it's easy to redefine it here
-#define emit
 
 
