@@ -17,27 +17,30 @@
 #include <qapplication.h>
 #include <qdir.h>
 #include <qstringlist.h>
-#include <q3listview.h>
+#include <QListWidget>
 #include <qlabel.h>
 #include <QMainWindow>
 #include <qstatusbar.h>
+#include <QtDebug>
 
 #include <dlfcn.h>	// for dl*()
 
 #include "extalgorithm.h"
 
+// NPlugin
 #include "pluginmanager.h"
 #include "plugincontainer.h"
 #include "ipluginuser.h"
 #include "iprovider.h"
-#include "pluginlistitem.h"
 #include "pluginsettingsdlg.h"
-#include "progressdisplaydlg.h"
-
-#include "ui_plugincontrol.h"
+#include "plugincontroldlgmeditator.h"
 
+// NXml
 #include "xmldata.h"
 
+// NUtil
+#include "progressdisplaydlg.h"
+
 #include "helpers.h"
 
 namespace NPlugin 
@@ -87,7 +90,7 @@ void PluginManager::loadPlugins()
 		// if the plugin is not disabled and no such plugin is already loaded
 		if (_disabledPlugins.find(it->name())==_disabledPlugins.end()  &&  it->pPlugin==0)
 		{
-			dlg._pPluginName->setText(toQString(it->name()));
+			dlg.setText("Loading " + toQString(it->name()));
 			dlg.setProgress(0);
 			qApp->processEvents();
 			loadPlugin(it->directory, it->name()); 
@@ -108,7 +111,8 @@ vector<PluginContainer*> PluginManager::getLoadedPlugins() const
 	for (map<string, PluginData>::const_iterator it = _plugins.begin(); 
 		it != _plugins.end(); ++it)
 	{
-		result.push_back(it->second.pPlugin);
+		if (it->second.pPlugin != 0)
+			result.push_back(it->second.pPlugin);
 	}
 	return result;
 }
@@ -146,7 +150,7 @@ vector<PluginManager::PluginData> PluginManager::getAvailablePlugins() const
 		// in the current directory, if both is possible the plugin is available
 		for (QStringList::iterator jt = pluginFiles.begin(); jt != pluginFiles.end(); ++jt)
 		{
-			qDebug("opening plugin file " + *jt);
+			qStrDebug("opening plugin file " + pluginDir.absolutePath() + "/" + *jt);
 			PluginData pluginData;
 			string libraryFile = toString(*jt);
 			string pluginName = libraryFile.substr(3,libraryFile.length() - 6);	// remove "lib" and ".so"
@@ -181,8 +185,19 @@ vector<PluginManager::PluginData> PluginManager::getAvailablePlugins() const
 PluginContainer* PluginManager::loadPlugin(
 	const string& directory, const string& pluginName)
 {
+	if (isLoaded(toQString(pluginName)))
+		return 0;
 	typedef PluginContainer* (*NewPluginFkt)();
 	typedef PluginInformation (*GetPluginDataFkt)();
+	
+	// holds if a progress dialog was available when calling this method
+	bool progressDialogAvailable = (_pProgressDlg != 0);
+	if (!progressDialogAvailable)
+	{
+		_pProgressDlg = new NUtil::ProgressDisplayDlg(_pProvider->mainWindow(), "PluginProgressDlg", true);
+// 		_pProgressDlg->show();
+	}
+	
 	PluginContainer* pPlugin = 0;
 	dlerror();	// dismiss pending errors
 	void* libraryHandle = dlopen((directory + "lib" + pluginName + ".so").c_str(), RTLD_LAZY);
@@ -195,7 +210,7 @@ PluginContainer* PluginManager::loadPlugin(
 		if (func != 0)	// if no error occured on load (i.e. the symbol was present)
 		{
 			pPlugin = func();	// fetch the plugin
-			qDebug("Loaded plugin: " + pPlugin->title());
+ 			qDebug() << "Loaded plugin: " << pPlugin->title();
 			pluginData.directory = directory;
 			pluginData.libraryHandle = libraryHandle;
 			GetPluginDataFkt getInformation = GetPluginDataFkt(
@@ -227,6 +242,11 @@ PluginContainer* PluginManager::loadPlugin(
 			dlerror();	// dismiss possible errors
 		}
 	}
+	if (!progressDialogAvailable)
+	{
+		delete _pProgressDlg;
+		_pProgressDlg = 0;
+	}
 	return pPlugin;
 }
 
@@ -249,7 +269,13 @@ void PluginManager::unloadPlugin(PluginContainer* pPlugin)
 	delete pPlugin;
 	it->second.pPlugin = 0;
 	dlclose(it->second.libraryHandle);
-	_pProvider->statusBar()->message("Unloaded plugin " + pluginTitle, 3000);
+	_pProvider->statusBar()->showMessage("Unloaded plugin " + pluginTitle, 3000);
+}
+
+void PluginManager::unloadPlugin(const string& pluginName)
+{
+	PluginToDataMap::iterator it = _plugins.find(pluginName);
+	unloadPlugin(it->second.pPlugin);
 }
 
 void PluginManager::saveSettings(NXml::XmlData& outData, QDomElement parent) const
@@ -270,7 +296,7 @@ void PluginManager::saveSettings(NXml::XmlData& outData, QDomElement parent) con
 		// if the plugin was loaded
 		if (pluginData.pPlugin != 0)
 		{
-			qDebug("Saving " + toQString(pluginData.name()));
+			qDebug() << "Saving " << pluginData.name().c_str();
 			pluginData.pPlugin->saveSettings(outData, pluginSettings);
 		}
 		// the plugin was not loaded
@@ -312,7 +338,7 @@ QDomElement PluginManager::loadSettings(QDomElement source)
 		QDomElement settings = NXml::getFirstElement(pluginSettings.firstChild());
 		while (!settings.isNull())
 		{
-			qDebug("Loaded " + settings.tagName() + " settings");
+// 			qDebug() << "Loaded " << settings.tagName() << " settings";
 			// the settings for the current plugin
 			_pluginSettings.insert( make_pair(toString(settings.tagName()),settings) );
 			settings = NXml::getNextElement(settings);
@@ -324,26 +350,13 @@ QDomElement PluginManager::loadSettings(QDomElement source)
 
 void PluginManager::showControlDialog(QWidget* pParent)
 {
-	QDialog dlg(pParent);
-	Ui::PluginControl uiDlg;
-	uiDlg.setupUi(&dlg);
-	_pControlDialog = &dlg;
-	_pPluginListView = uiDlg._pPluginList;
-	vector<PluginData> plugins = getAvailablePlugins();
-	for (vector<PluginData>::iterator it = plugins.begin(); it != plugins.end(); ++it)
-	{
-		PluginListItem* pItem = new PluginListItem(uiDlg._pPluginList, *it);
-		connect( pItem, SIGNAL(toggled(PluginListItem*, bool)), 
-			SLOT(onPluginToggled(PluginListItem*, bool)) );
-	}
-	dlg.exec();
-	_pPluginListView = 0;
-	_pControlDialog = 0;
+	PluginControlDlgMeditator mediator(this);
+	mediator.displayDialog(pParent);
 }
 
 void PluginManager::showSettingsDialog(QWidget* pParent)
 {
-	PluginSettingsDlg dlg;
+	PluginSettingsDlg dlg(pParent);
 	vector<PluginContainer*> plugins = getLoadedPlugins();
 	for (vector<PluginContainer*>::iterator it = plugins.begin(); it != plugins.end(); ++it)
 	{
@@ -364,46 +377,19 @@ bool PluginManager::checkDlError() const
 	return false;
 }
 
-void PluginManager::onPluginToggled(PluginListItem* pSrc, bool state)
-{
-	const PluginData& pd = pSrc->pluginData();
-	if (state)
-	{
-		assert(pd.libraryHandle==0);
-		assert(pd.pPlugin==0);
-		NUtil::ProgressDisplayDlg dlg(_pControlDialog, "PluginProgressDlg", true);
-		_pProgressDlg = &dlg;
-		dlg.show();
-		PluginContainer* pPlugin = loadPlugin(pd.directory, pd.information.name);
-		_pProgressDlg = 0;
-		if (pPlugin)
-		{
-			const PluginData& newData = _plugins[pPlugin->name()];
-			pSrc->setDynamicData(newData.libraryHandle, pPlugin);
-		}
-		else
-		{
-			qDebug("Error loading plugin");
-			// ignore this state change
-			disconnect( pSrc, SIGNAL(toggled(PluginListItem*, bool)), 
-				this, SLOT(onPluginToggled(PluginListItem*, bool)) );
-			pSrc->setState(Q3CheckListItem::Off);
-			connect( pSrc, SIGNAL(toggled(PluginListItem*, bool)), 
-				SLOT(onPluginToggled(PluginListItem*, bool)) );
-		}
-	}
-	else
-	{
-		assert(pd.libraryHandle);
-		assert(pd.pPlugin);
-		unloadPlugin(pd.pPlugin);
-		pSrc->setDynamicData(0, 0);
-	}
-}
-
 NUtil::IProgressObserver* PluginManager::progressObserver()
 {
 	return _pProgressDlg;
 }
 
+
+bool PluginManager::isLoaded(QString pluginName) const
+{
+	PluginToDataMap::const_iterator it = _plugins.find(toString(pluginName));
+	if ( it != _plugins.end() && it->second.pPlugin != 0 )
+		return true;
+	return false;
+}
+
+
 };
