@@ -33,7 +33,7 @@
 using namespace std;
 
 class QWidget;
-class Q3ListView;
+class QTableWidget;
 class PluginProgressDlg;
 class QDialog;
 
@@ -69,16 +69,19 @@ class PluginListItem;
   * (i.e. neither disabled nor active) its settings will not be saved even
   * if they were loaded before.
   *
+  * Only one plugin of a given name can be loaded at a time, however multiple 
+  * plugins with that name can be available in different directories.
+  *
   * @author Benjamin Mesing
   */
 class PluginManager : public QObject, public PluginInformer, public NXml::IXmlStorable, public IPluginUser
 {
 	Q_OBJECT
+	
+	friend class PluginControlDlgMeditator;
 private:
 	typedef vector<IPluginUser*> PluginUserContainer;
 	PluginUserContainer _users;
-private slots:
-	void onPluginToggled(PluginListItem* pSrc, bool state);
 protected:
 
 	/** @brief This holds information about the plugin. */
@@ -160,12 +163,15 @@ protected:
 	  * which are loaded but not longer in any directory.
 	  */
 	vector<PluginData> getAvailablePlugins() const;
+	
 	/** @brief This loads the plugin and informs the plugin users about the loading. 
 	  *
-	  * The plugin will be added to the list of loaded plugins.
+	  * The plugin will be added to the list of loaded plugins. If a plugin with
+	  * the same pluginName is already loaded, loading will be aborted.
 	  * @param directory the directory where the plugin is located
 	  * @param pluginName the name of the plugin
-	  * @returns a pointer to the plugin loaded, 0 if loading failed
+	  * @returns a pointer to the plugin loaded, 0 if loading failed or a plugin of
+	  * the given name was already loaded.
 	  * @note You should have called loadSettings before because else
 	  * this function will be unloaded. 
 	  */
@@ -175,9 +181,17 @@ protected:
 	  * It will delete the plugin and close the library, so make sure that every reference
 	  * is removed in the plugin users.
 	  * @param pPlugin the plugin to be unloaded 
-	  * @pre pPlugin is_in keys(_loadedPlugins)
+	  * @pre the plugin must be loaded
 	  */
 	void unloadPlugin(PluginContainer* pPlugin);
+	/** @brief This unloads the plugin and informs the plugin users about the unloading. 
+	  * 
+	  * It will delete the plugin and close the library, so make sure that every reference
+	  * is removed in the plugin users.
+	  * @param pluginName the name of the plugin to be unloaded 
+	  * @pre the plugin must be loaded
+	  */
+	void unloadPlugin(const string& pluginName);
 private:
 	
 	friend class PluginListItem;
@@ -216,6 +230,11 @@ private:
 	bool checkDlError() const;
 	
 	typedef map<string, PluginData> PluginToDataMap;
+	/** @brief This datastructure maps all plugins that were loaded at least once 
+	  * to their plugin data.
+	  *
+	  * When plugins are unloaded the will not be removed from this map.
+	  */
 	PluginToDataMap _plugins;
 	
 	
@@ -233,7 +252,7 @@ private:
  	set<string> _disabledPlugins;
 	/** Holds the listview where the plugins are shown to the user, only set
 	  * while the plugin dialog is shown.  */
-	Q3ListView* _pPluginListView;
+	QTableWidget* _pPluginDisplay;
 public:
 	/**
 	  * 
@@ -244,6 +263,9 @@ public:
 	PluginManager(vector<string> directories, IProvider* pProvider);
 	~PluginManager();
 
+	/** Returns true if a plugin with the given name is already loaded, else false. */
+	bool isLoaded(QString name) const;
+
 	/** @brief Save the settings from this plugin container into the given XML tree
 	  * 
 	  * Currently the only information is the plugins which are disabled.
