@@ -14,6 +14,9 @@
 #include <qtabwidget.h>
 #include <qlabel.h>
 #include <qpushbutton.h>
+#include <QVBoxLayout>
+#include <QHBoxLayout>
+
 
 #include "pluginsettingsdlg.h"
 
@@ -24,19 +27,24 @@ namespace NPlugin
 {
 
 PluginSettingsDlg::PluginSettingsDlg(QWidget *parent, const char *name)
- : QDialog(parent, name)
+ : QDialog(parent)
 {
-	new QVBoxLayout(this, 5, 6);
-	_pPluginWidget = new QTabWidget(this, "PluginSettingsTabs");
-	layout()->add(_pPluginWidget);
-	QHBoxLayout* pButtonLayout = new QHBoxLayout(layout(), -1, "ButtonLayout");
-	QButton* pOkButton = new QPushButton(tr("&OK"), this, "OkButton");
+	setObjectName(name);
+	QVBoxLayout* pMainLayout = new QVBoxLayout(this);
+	pMainLayout->setMargin(5);
+	pMainLayout->setSpacing(6);
+	_pPluginWidget = new QTabWidget(this);
+	_pPluginWidget->setObjectName("PluginSettingsTabs");
+	pMainLayout->addWidget(_pPluginWidget);
+	QHBoxLayout* pButtonLayout = new QHBoxLayout();
+	pMainLayout->addLayout(pButtonLayout);
+	QPushButton* pOkButton = new QPushButton(tr("&OK"), this);
 	connect(pOkButton, SIGNAL(clicked()), SLOT(accept()));
-	QButton* pCancelButton = new QPushButton(tr("&Cancel"), this, "CancleButton");
+	QPushButton* pCancelButton = new QPushButton(tr("&Cancel"), this);
 	connect(pCancelButton, SIGNAL(clicked()), SLOT(reject()));
 	pButtonLayout->addItem(new QSpacerItem(0,0, QSizePolicy::Expanding, QSizePolicy::Minimum ));
-	pButtonLayout->add(pOkButton);
-	pButtonLayout->add(pCancelButton);
+	pButtonLayout->addWidget(pOkButton);
+	pButtonLayout->addWidget(pCancelButton);
 }
 
 
